/*******************************************************
 *
 *  File: cpu.c
 *
 *  Author: Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *  (c) Copyright 2001, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************/

#include "cpu.h"
#include "defs.h"


/*
 * This is the memory map set up by the boot monitor
 * Physical      Virtual
 * ========================
 * 0xF0000000 -> 0x00000000
 * 0xF1000000 -> 0x00400000
 * 0xF4000000 -> 0x00800000
 * 0xF5000000 -> 0x00C00000
 * 0xF8000000 -> 0x01000000
 * 0xF9000000 -> 0x01400000
 * 0xFC000000 -> 0x01800000
 * 0xFD000000 -> 0x01C00000  (Not user-writeable, not buffered)
 *
 * 256MB starting from physical 0xf0000000 is mapped at virtual
 * 0xf0000000 as well.
 *
 * The MMU is still on when ak2boot.srec is started.
 */



#if 0

#define FIRST_LEVEL_TABLE 0xf1000000

static void memory_map_1MB_chunk(unsigned phys, unsigned virt, unsigned chunks, int cache, int buf)
{
	int 		value;
	unsigned*	entry = ((unsigned*) FIRST_LEVEL_TABLE) + (virt >> 20);
	unsigned 	cb = (cache ? BIT3 : 0) | (buf? BIT2 : 0);

	while(chunks--)
	{
		value = (phys >> 20) << 20;
		*entry++ = ((value) & ~(BIT2 | BIT3)) | cb;

		phys += (1<<20);
	}
}


static void first_level_table_mark_cb(int offset, int cache, int buf)
{
	unsigned*	entry = ((unsigned*) FIRST_LEVEL_TABLE) + offset;
	unsigned 	cb = (cache ? BIT3 : 0) | (buf? BIT2 : 0);

	*entry = ((*entry) & ~(BIT2 | BIT3)) | cb;
}

static void first_level_table_setup()
{
	unsigned*	entry = (unsigned*) FIRST_LEVEL_TABLE;
	unsigned	desc;
	int			i;

		/*
		 * Map entire 4GB as linear == physical
		 * Default is not to buffer or cache
		 */
	for(i=0 ; i<4096 ; i++)
	{
		desc = 	BIT1 							// section descriptor
				|  BIT5 | BIT6 | BIT7 | BIT8 	// All domains
				| BIT10 | BIT11					// read/write access
				;
		desc |= (i<<20);
		*entry++ = desc;
	}
	memory_map_1MB_chunk(0xfc000000, 0x01800000, 4, 1, 1);

	_cp15_write_translation_table_base(FIRST_LEVEL_TABLE);
	_cp15_write_domain_access_control(0xffffffff);
}
#endif


void cpu_init_stuff()
{
	unsigned	cpsr;

		/*
		 * disable interrupts and set supervisor mode
		 */
	cpsr = _cpsr_read();
	cpsr &=  ~(0x1f);				// clear mode
	cpsr |=  0xd3;					// disable IRQs and FIQs, SVC mode
	_cpsr_write(cpsr);

		/*
		 * We seem to need this !?
		 */
	_cp15_write_domain_access_control(0xffffffff);
}


