/*******************************************************
 *
 *  File: lib.c
 *
 *  Author: Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *  (c) Copyright 2001, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************/

#include "lib.h"

/*******************************************
 *
 * Implementation functions
 *
 *******************************************/

static char* my_strcpy(char* dst, const char* src, int original )
{
	char*	tmp = dst;
	while(*src)
		*dst++ = *src++;
	*dst = 0;
	return original ? tmp : dst;
}


static char* my_itoa(int i, char* str, int original)
{
	int		digit;
	int		digits = 0;
	int		div = 1000000000;
	char*	s = str;

	if (i<0)
	{
		*str++ = '-';
		i = i * -1;
	}
	*str = '0';
	while(div>0)
	{
		digit = (i / div);
		if (digit || digits)
		{
			digits++;
			*str++ = digit+'0';
			i -= digit * div;
		}
		div /= 10;
	}
	if (digits == 0)
		str++;
	*str = 0;
	
	return original ? s : str;
}

static char* my_itoa_hex(unsigned num, char* str, int original)
{
	int		i, count;
	int		digit;
	char*	s = str;
	
	*str++ = '0';
	*str++ = 'x';
	count = 0;
	for(i=0 ; i<8 ; i++)
	{
		digit = num >> 28;
		if (digit || count)
		{
			*str++ = (digit < 10) ? digit + '0' : (digit - 10) + 'A';
			count++;
		}
		num <<= 4;
	}
	if (!count)
		*str++ = '0';
	*str = 0;
	
	return original ? s : str;
}


/*******************************************
 *
 * Interface functions
 *
 *******************************************/

int lib_strlen(const char* str)
{
	int	len = 0;
	while(*str++)
		len++;
	return len;
}


char* lib_strcpy(char* dst, const char* src)
{
	return my_strcpy(dst, src, 1);
}

char* lib_itoa(int i, char* str)
{
	return my_itoa(i, str, 1);
}

char* lib_itoa_hex(unsigned num, char* str)
{
	return my_itoa_hex(num, str, 1);
}

char* lib_strcat(char* dst, const char* src)
{
	char* tmp = dst;
	while(*dst++)
		;
	lib_strcpy(dst-1, src);
	return tmp;
}

int lib_sprintf(char* buf, const char* fmt, ...)
{
	va_list args;
	int i;
	va_start(args, fmt);
	i = lib_vsprintf(buf, fmt, args);
	va_end(args);
	return i;
}

int lib_vsprintf(char* buf, const char* fmt, va_list args)
{
	char		c;
	unsigned	hex_num;
	int			num;
	char*		s;
	
	while( (c=*fmt++) )
	{
		if (c == '%')
		{
			c = *fmt++;
			switch (c)
			{
				case 'c':
					num = va_arg(args, int);
					*buf++ = (char) num;
					break;
				case 's':
					s = va_arg(args, char*);
					buf = my_strcpy(buf, s, 0);
					break;
				case 'd':
					num = va_arg(args, int);
					buf = my_itoa(num, buf, 0);
					break;
				case 'x':
					hex_num = va_arg(args, unsigned);
					buf = my_itoa_hex(hex_num, buf, 0);
					break;
				case '%':
				default:
					*buf++ = c;
					break;
			}
		}
		else
		{
			*buf++ = c;
		}
	}
	*buf = 0;
	return 0;
}


unsigned lib_bits(unsigned reg, int first, int last)
{
	unsigned 	mask = 0;
	int			i;
	
	for (i=first ; i<last+1 ; i++)
		mask |= (1 << i);
	return (reg & mask) >> first;
	
}

unsigned lib_bit(unsigned reg, int bit)
{
	return lib_bits(reg, bit, bit);
}

char* lib_my_atoi(char* buf, int* i)
{
	int		neg = 0;
	int		num = 0;
	
	if (*buf == '-')
	{
		neg = 1;
		buf++;
	}
	while((*buf >= '0') && (*buf <= '9'))
	{
		num *= 10;
		num += *buf - '0';
		buf++;
	}
	if (neg)
		num *= -1;
	*i = num;
	
	return buf;
}

char* lib_memcpy(char* dst, const char* src, int len)
{
	char* temp = dst;
	while(len--)
		*dst++ = *src++;
	return temp;
}

int lib_strncmp(const char* s1, const char* s2, int len)
{
	if (s1 == NULL)
		return -1;
	if (s2 == NULL)
		return 1;
	while(len && *s1 && *s2)
	{
		if (*s1 != *s2)
			return (*s1 < *s2);
		s1++;
		s2++;
		len--;
	}
	if (len == 0)
		return 0;
	if (*s1 == 0)
		return -1;
	return 1;
}
