/*******************************************************
 *
 *  File: uart.h
 *
 *  Author: Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *  (c) Copyright 2001, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************/

#ifndef __UART_H
#define __UART_H

#include "defs.h"

typedef unsigned long U32;


#define UART1_BASE								0x80000600
#define UART2_BASE								0x80000700
#define UART3_BASE								0x80000800

const char* sUarts[] =
{
	0,
	(char*)UART1_BASE,
	(char*)UART2_BASE,
	(char*)UART3_BASE,
};

#define UART_UARTDR(uart_base)							(uart_base + 0x00)
#define UART_UARTFCR(uart_base)							(uart_base + 0x04)
#define UART_UARTBRCR(uart_base)						(uart_base + 0x08)
#define UART_UARTCON(uart_base)							(uart_base + 0x0C)
#define UART_UARTFLG(uart_base)							(uart_base + 0x10)
#define UART_UARTINT(uart_base)							(uart_base + 0x14)
#define UART_UARTINTM(uart_base)						(uart_base + 0x18)
#define UART_UARTINTRES(uart_base)						(uart_base + 0x1C)


#define REG_UART_UARTDR(uart)							*(( volatile U32 *)(UART_UARTDR(sUarts[uart])))
#define REG_UART_UARTFCR(uart)							*(( volatile U32 *)(UART_UARTFCR(sUarts[uart])))
#define REG_UART_UARTBRCR(uart)							*(( volatile U32 *)(UART_UARTBRCR(sUarts[uart])))
#define REG_UART_UARTCON(uart)							*(( volatile U32 *)(UART_UARTCON(sUarts[uart])))
#define REG_UART_UARTFLG(uart)							*(( volatile const U32 *)(UART_UARTFLG(sUarts[uart])))
#define REG_UART_UARTINT(uart)							*(( volatile U32 *)(UART_UARTINT(sUarts[uart])))
#define REG_UART_UARTINTM(uart)							*(( volatile U32 *)(UART_UARTINTM(sUarts[uart])))
#define REG_UART_UARTINTRES(uart)						*(( volatile const U32 *)(UART_UARTINTRES(sUarts[uart])))

/* uartFlg */
#define UARTFLG_TXFE				BIT7
#define UARTFLG_RXFF				BIT6
#define UARTFLG_TXFF				BIT5
#define UARTFLG_RXFE				BIT4
#define UARTFLG_BUSY				BIT3
#define UARTFLG_DCD					BIT2
#define UARTFLG_DSR					BIT1
#define UARTFLG_CTS					BIT0

#define UARTDR_BE					BIT11
#define UARTDR_OE					BIT10
#define UARTDR_PE					BIT9
#define UARTDR_FE					BIT8
#define UARTDR_DATA					(0xff)

#endif		/* __UART_H */


